import datetime
import logging
import re

import pytz
from gunicorn.glogging import Logger

# Gunicorn configuration file.
#
# Server socket
#
#   bind - The socket to bind.
#
#       A string of the form: 'HOST', 'HOST:PORT', 'unix:PATH'.
#       An IP is a valid HOST.
#
#   backlog - The number of pending connections. This refers
#       to the number of clients that can be waiting to be
#       served. Exceeding this number results in the client
#       getting an error when attempting to connect. It should
#       only affect servers under significant load.
#
#       Must be a positive integer. Generally set in the 64-2048
#       range.
#

bind = "0.0.0.0:8080"
backlog = 2048

#
# Worker processes
#
#   workers - The number of worker processes that this server
#       should keep alive for handling requests.
#
#       A positive integer generally in the 2-4 x $(NUM_CORES)
#       range. You'll want to vary this a bit to find the best
#       for your particular application's work load.
#
#   worker_class - The type of workers to use. The default
#       sync class should handle most 'normal' types of work
#       loads. You'll want to read
#       http://docs.gunicorn.org/en/latest/design.html#choosing-a-worker-type
#       for information on when you might want to choose one
#       of the other worker classes.
#
#       A string referring to a Python path to a subclass of
#       gunicorn.workers.base.Worker. The default provided values
#       can be seen at
#       http://docs.gunicorn.org/en/latest/settings.html#worker-class
#
#   worker_connections - For the eventlet and gevent worker classes
#       this limits the maximum number of simultaneous clients that
#       a single process can handle.
#
#       A positive integer generally set to around 1000.
#
#   timeout - If a worker does not notify the master process in this
#       number of seconds it is killed and a new worker is spawned
#       to replace it.
#
#       Generally set to thirty seconds. Only set this noticeably
#       higher if you're sure of the repercussions for sync workers.
#       For the non sync workers it just means that the worker
#       process is still communicating and is not tied to the length
#       of time required to handle a single request.
#
#   keepalive - The number of seconds to wait for the next request
#       on a Keep-Alive HTTP connection.
#
#       A positive integer. Generally set in the 1-5 seconds range.
#

workers = 1
worker_class = "sync"
worker_connections = 1000
timeout = 30
keepalive = 2

#
#   spew - Install a trace function that spews every line of Python
#       that is executed when running the server. This is the
#       nuclear option.
#
#       True or False
#

spew = False

#
# Server mechanics
#
#   daemon - Detach the main Gunicorn process from the controlling
#       terminal with a standard fork/fork sequence.
#
#       True or False
#
#   raw_env - Pass environment variables to the execution environment.
#
#   pidfile - The path to a pid file to write
#
#       A path string or None to not write a pid file.
#
#   user - Switch worker processes to run as this user.
#
#       A valid user id (as an integer) or the name of a user that
#       can be retrieved with a call to pwd.getpwnam(value) or None
#       to not change the worker process user.
#
#   group - Switch worker process to run as this group.
#
#       A valid group id (as an integer) or the name of a user that
#       can be retrieved with a call to pwd.getgrnam(value) or None
#       to change the worker processes group.
#
#   umask - A mask for file permissions written by Gunicorn. Note that
#       this affects unix socket permissions.
#
#       A valid value for the os.umask(mode) call or a string
#       compatible with int(value, 0) (0 means Python guesses
#       the base, so values like "0", "0xFF", "0022" are valid
#       for decimal, hex, and octal representations)
#
#   tmp_upload_dir - A directory to store temporary request data when
#       requests are read. This will most likely be disappearing soon.
#
#       A path to a directory where the process owner can write. Or
#       None to signal that Python should choose one on its own.
#

daemon = False
raw_env = [
    # 'SPAM=eggs',
]
pidfile = None
umask = 0
user = None
group = None
tmp_upload_dir = None

#
#   Logging
#
#   logfile - The path to a log file to write to.
#
#       A path string. "-" means log to stdout.
#
#   loglevel - The granularity of log output
#
#       A string of "debug", "info", "warning", "error", "critical"
#


class CustomLogger(Logger):
    def now(self):
        return datetime.datetime.now(tz=pytz.timezone("Europe/London")).strftime("%d-%b-%y %H:%M:%S")


logger_class = CustomLogger

errorlog = "-"
loglevel = "info"
accesslog = "-"
access_log_format = '{"logType": "gunicorn-access", "remote_ip":"%({x-forwarded-for}i)s","request_id":"%({X-Amzn-Trace-Id}i)s","response_code":"%(s)s","request_method":"%(m)s","request_path":"%(U)s","request_querystring":"%(q)s","request_timetaken":"%(D)s","response_length":"%(B)s"}'  # noqa

#
# Process naming
#
#   proc_name - A base to use with setproctitle to change the way
#       that Gunicorn processes are reported in the system process
#       table. This affects things like 'ps' and 'top'. If you're
#       going to be running more than one instance of Gunicorn you'll
#       probably want to set a name to tell them apart. This requires
#       that you install the setproctitle module.
#
#       A string or None to choose a default of something like 'gunicorn'.
#

proc_name = None

#
# Server hooks
#
#   post_fork - Called just after a worker has been forked.
#
#       A callable that takes a server and worker instance
#       as arguments.
#
#   pre_fork - Called just prior to forking the worker subprocess.
#
#       A callable that accepts the same arguments as after_fork
#
#   pre_exec - Called just prior to forking off a secondary
#       master process during things like config reloading.
#
#       A callable that takes a server instance as the sole argument.
#


def post_fork(server, worker):
    server.log.info("Worker spawned (pid: %s)", worker.pid)


def pre_fork(server, worker):
    pass


def pre_exec(server):
    server.log.info("Forked child, re-executing.")


def when_ready(server):
    server.log.info("Server is ready. Spawning workers")


def worker_int(worker):
    worker.log.info("worker received INT or QUIT signal")

    # get traceback info
    import sys
    import threading
    import traceback

    id2name = {th.ident: th.name for th in threading.enumerate()}
    code = []
    for threadId, stack in sys._current_frames().items():
        code.append("\n# Thread: %s(%d)" % (id2name.get(threadId, ""), threadId))
        for filename, lineno, name, line in traceback.extract_stack(stack):
            code.append('File: "%s", line %d, in %s' % (filename, lineno, name))
            if line:
                code.append("  %s" % (line.strip()))
    worker.log.debug("\n".join(code))


def worker_abort(worker):
    worker.log.info("worker received SIGABRT signal")


class RequestPathFilter(logging.Filter):
    def __init__(self, *args, path_re, **kwargs):
        super().__init__(*args, **kwargs)
        self.path_filter = re.compile(path_re)

    def filter(self, record):
        req_path = record.args["U"]
        if not self.path_filter.match(req_path):
            return True  # log this entry
        # ... additional conditions can be added here ...
        return False  # do not log this entry


def on_starting(server):
    server.log.access_log.addFilter(RequestPathFilter(path_re=r"^/healthcheck$"))



import os
from unittest import mock

import pytest

from fsd_utils.config.commonconfig import CommonConfig


@pytest.mark.parametrize(
    "flask_env, env_secret_key, exp_secret_key",
    [
        ("dev", "dev_key", "dev_key"),
        ("development", "dev_key", "dev_key"),
        ("uat", "abc123", "abc123"),
        ("development", "", "dev-secret"),
        ("unit_test", "", "dev-secret"),
        ("prod", "prod_secret", "prod_secret"),
    ],
)
def test_config(flask_env, env_secret_key, exp_secret_key):
    with mock.patch.dict(os.environ, {"SECRET_KEY": env_secret_key}):
        result = CommonConfig._resolve_secret_key(flask_env)
    assert result == exp_secret_key


@pytest.mark.parametrize(
    "flask_env, env_secret_key",
    [
        (
            "dev",
            "",
        ),
        (
            "test",
            "",
        ),
        (
            "uat",
            "",
        ),
        (
            "production",
            "",
        ),
        ("db_migrations", ""),
        ("prod", ""),
    ],
)
def test_config_error(flask_env, env_secret_key):
    with mock.patch.dict(os.environ, {"SECRET_KEY": env_secret_key}):
        with pytest.raises(KeyError):
            CommonConfig._resolve_secret_key(flask_env)



import requests
from flask import current_app


def get_remote_data_as_json(endpoint):
    response = requests.get(endpoint)
    if response.status_code == 200:
        data = response.json()
        return data
    else:
        current_app.logger.warning(
            "GET remote data call was unsuccessful with status code: %s body: %s.", response.status_code, response.text
        )
        response.raise_for_status()



from bs4 import BeautifulSoup
from flask import current_app


class FreeText:
    indent = " " * 5

    @classmethod
    def remove_html_tags(cls, answer):  # noqa: C901
        """
        Removes HTML tags from the provided answer and returns the cleaned text.

        Args:
            answer (str): The answer containing HTML tags.

        Returns:
            str: The cleaned text with HTML tags removed.
        """

        try:
            if answer is None or isinstance(answer, (bool, list)):
                return answer

            # Check if answer looks like a URL
            if answer.startswith("http://") or answer.startswith("https://"):
                return answer

            soup = BeautifulSoup(answer, "html.parser")

            if not soup.find():
                return answer.strip()

            if soup.ol:
                ol_tags = soup.find_all("ol")
                for ol in ol_tags:
                    li_tags = ol.find_all("li")
                    for index, li in enumerate(li_tags, start=1):
                        if li.get_text() == "\xa0":
                            continue
                        li.replace_with(f"{cls.indent}{str(index)}. {li.get_text()}\n")

            if soup.ul:
                ul_tags = soup.find_all("ul")
                for ul in ul_tags:
                    li_tags = ul.find_all("li")
                    for _, li in enumerate(li_tags, start=1):
                        if li.get_text() == "\xa0":
                            continue
                        li.replace_with(f"{cls.indent}. {li.get_text()}\n")

            plain_text = soup.get_text().strip()
            return plain_text

        except Exception:
            current_app.logger.error("Error occurred while processing HTML tag: %s", answer)
            return answer



import json
import unittest
from datetime import datetime
from unittest.mock import MagicMock, call, patch

from fsd_utils.services.aws import SQSClient


class TestSQSClient(unittest.TestCase):
    def setUp(self):
        # Create a mock Boto3 SQS client for testing
        self.sqs_client = MagicMock()
        self.sqs_client.list_queues.return_value = {"QueueUrls": ["url1", "url2"]}
        self.sqs_client.delete_queue.return_value = {}

        # Create an instance of SQSClient with the mock client
        self.sqs = SQSClient("your_access_key", "your_secret_key")
        self.sqs.client = self.sqs_client

    def test_get_queues(self):
        # Test the get_queues method without a prefix
        queue_names = self.sqs.get_queues()
        self.assertEqual(queue_names, ["url1", "url2"])

    def test_remove_queue_success(self):
        # Test the remove_queue method when the queue is successfully deleted
        queue_url = "test_queue_url"
        self.sqs.remove_queue(queue_url)
        self.sqs_client.delete_queue.assert_called_with(QueueUrl=queue_url)

    def test_remove_queue_failure(self):
        # Test the remove_queue method when there is an error
        queue_url = "test_queue_url"
        self.sqs_client.delete_queue.side_effect = Exception("Queue deletion failed")
        with self.assertRaises(Exception):  # noqa: B017
            self.sqs.remove_queue(queue_url)

    def test_get_queue_url(self):
        # Mock data & responses
        queue_name = "test_queue"
        expected_queue_url = "https://sqs.us-west-1.amazonaws.com/123456789012/test_queue"
        self.sqs_client.get_queue_url.return_value = {"QueueUrl": expected_queue_url}

        # call to the function
        actual_queue_url = self.sqs.get_queue_url(queue_name)

        # Assert responses
        self.assertEqual(actual_queue_url, expected_queue_url)
        self.sqs_client.get_queue_url.assert_called_with(QueueName=queue_name)

    def test_submit_single_message(self):
        # Mock data & responses
        queue_url = "https://sqs.us-west-1.amazonaws.com/123456789012/test_queue"
        message = {"key": "value"}
        message_group_id = "group_id"
        message_deduplication_id = "deduplication_id"
        expected_message_id = "test_message_id"

        self.sqs_client.send_message.return_value = {"MessageId": expected_message_id}

        with patch("fsd_utils.services.aws.datetime") as mock_datetime:
            datetime_now = datetime(2023, 1, 1, 12, 0, 0)
            mock_datetime.now.return_value = datetime_now

            # call to the function
            actual_message_id = self.sqs.submit_single_message(
                queue_url,
                message,
                message_group_id=message_group_id,
                message_deduplication_id=message_deduplication_id,
            )

            # Assert responses
            self.assertEqual(actual_message_id, expected_message_id)
            self.sqs_client.send_message.assert_called_with(
                QueueUrl=queue_url,
                MessageBody=json.dumps(message),
                MessageAttributes={
                    "message_created_at": {
                        "StringValue": str(datetime_now),
                        "DataType": "String",
                    }
                },
                MessageGroupId=message_group_id,
                MessageDeduplicationId=message_deduplication_id,
            )

    def test_submit_message(self):
        # Mock data & responses
        queue_url = "http://localhost:4576/queue/test_queue"
        messages = [
            {"body": {"key1": "value1"}, "attributes": {"attr1": "attr_value1"}},
            {"body": {"key2": "value2"}, "attributes": {"attr2": "attr_value2"}},
            {"body": {"key3": "value3"}, "attributes": {"attr3": "attr_value3"}},
            {"body": {"key4": "value4"}, "attributes": {"attr4": "attr_value4"}},
        ]
        expected_response = {
            "Successful": [{"MessageId": "msg1"}, {"MessageId": "msg2"}],
            "Failed": [{"Id": "2"}, {"Id": "3"}],
        }
        self.sqs_client.send_message_batch.return_value = expected_response

        # call to the function
        actual_response = self.sqs.submit_message(queue_url, messages, DelaySeconds=1)

        # Assert responses
        self.assertEqual(actual_response, expected_response)

    def test_receive_messages(self):
        # Mock data & responses
        queue_url = "http://localhost:4576/queue/test_queue"
        max_number = 5
        visibility_time = 10
        wait_time = 2
        messages = [
            {
                "MessageId": "msg1",
                "MessageAttributes": {"attr1": "value1"},
                "MessageBody": "Message 1",
            },
            {
                "MessageId": "msg2",
                "MessageAttributes": {"attr2": "value2"},
                "MessageBody": "Message 2",
            },
        ]
        response = {"Messages": messages}
        self.sqs_client.receive_message.return_value = response

        # call to the function
        received_messages = self.sqs.receive_messages(queue_url, max_number, visibility_time, wait_time)

        # Assert responses
        self.assertEqual(received_messages, messages)
        self.sqs_client.receive_message.assert_called_with(
            QueueUrl=queue_url,
            AttributeNames=["SentTimestamp", "ApproximateReceiveCount"],
            MessageAttributeNames=["All"],
            MaxNumberOfMessages=max_number,
            VisibilityTimeout=visibility_time,
            WaitTimeSeconds=wait_time,
        )

    def test_delete_messages(self):
        # Mock data & responses
        queue_url = "http://localhost:4576/queue/test_queue"
        message_receipt_handles = [
            "receipt_handle1",
            "receipt_handle2",
            "receipt_handle3",
        ]
        response = {
            "Successful": [{"Id": "0"}, {"Id": "1"}],
            "Failed": [{"Id": "2"}],
        }
        self.sqs_client.delete_message_batch.return_value = response

        # call to the function
        deleted_response = self.sqs.delete_messages(queue_url, message_receipt_handles)

        # Assert responses
        self.assertEqual(deleted_response, response)
        expected_entries = [
            {"Id": "0", "ReceiptHandle": "receipt_handle1"},
            {"Id": "1", "ReceiptHandle": "receipt_handle2"},
            {"Id": "2", "ReceiptHandle": "receipt_handle3"},
        ]
        self.sqs_client.delete_message_batch.assert_called_with(QueueUrl=queue_url, Entries=expected_entries)

    def test_create_sqs_queue(self):
        # Mock data & responses
        queue_name = "test_queue"
        has_dlq = True
        dlq_queue_name = "test_dlq"
        dlq_arn = "arn:aws:sqs:us-west-1:123456789012:test_dlq"
        max_receive_count = 3
        queue_url = "http://localhost:4576/queue/test_queue"
        dlq_queue_url = "http://localhost:4576/queue/test_dlq"
        self.sqs_client.get_queues.return_value = ["existing_queue"]
        self.sqs_client.create_queue.side_effect = [
            {"QueueUrl": queue_url},
            {"QueueUrl": dlq_queue_url},
        ]
        self.sqs_client.get_queue_url.return_value = {"QueueUrl": queue_url}
        self.sqs_client.get_queue_attributes.return_value = {"Attributes": {"QueueArn": dlq_arn}}

        # call to the function
        sqs_queue_url = self.sqs.create_sqs_queue(queue_name, has_dlq, dlq_queue_name, max_receive_count)

        # Assert responses
        self.assertEqual(sqs_queue_url, queue_url)
        self.sqs_client.create_queue.assert_has_calls(
            [
                call(QueueName=queue_name),
                call(QueueName=dlq_queue_name),
            ],
            any_order=True,
        )
        expected_redrive_policy = {
            "deadLetterTargetArn": dlq_arn,
            "maxReceiveCount": max_receive_count,
        }
        self.sqs_client.set_queue_attributes.assert_called_with(
            QueueUrl=queue_url,
            Attributes={"RedrivePolicy": json.dumps(expected_redrive_policy)},
        )

    def test_set_queue_attributes_with_queue_name(self):
        # Mock data & responses
        queue_name = "test_queue"
        queue_url = "http://localhost:4576/queue/test_queue"
        attributes = {"VisibilityTimeout": "10", "MaximumMessageSize": "1024"}
        self.sqs_client.get_queue_url.return_value = {"QueueUrl": queue_url}

        # call to the function
        self.sqs.set_queue_attributes(queue_name=queue_name, attributes=attributes)

        # Assert responses
        self.sqs_client.set_queue_attributes.assert_called_with(
            QueueUrl=queue_url,
            Attributes=attributes,
        )

    def test_set_queue_attributes_with_queue_url(self):
        # Mock data & responses
        queue_url = "http://localhost:4576/queue/test_queue"
        attributes = {"VisibilityTimeout": "10", "MaximumMessageSize": "1024"}

        # call to the function
        self.sqs.set_queue_attributes(queue_url=queue_url, attributes=attributes)

        # Assert responses
        self.sqs_client.set_queue_attributes.assert_called_with(
            QueueUrl=queue_url,
            Attributes=attributes,
        )



from flask import Response, current_app, make_response, redirect, request

from fsd_utils import CommonConfig


class LanguageSelector:
    @staticmethod
    def get_cookie_domain(cookie_domain):
        if not cookie_domain:
            return None
        else:
            return cookie_domain

    @staticmethod
    def set_language_cookie(locale: str, response: Response):
        response.set_cookie(
            CommonConfig.FSD_LANG_COOKIE_NAME,
            locale,
            domain=LanguageSelector.get_cookie_domain(current_app.config["COOKIE_DOMAIN"]),
            max_age=86400 * 30,  # 30 days
        )

    def __init__(self, app):
        self.flask_app = app
        self.flask_app.add_url_rule("/language/<locale>", view_func=self.select_language)

    @staticmethod
    def select_language(locale):
        # TODO: Perform additional validation on referrer
        response = make_response(redirect(request.referrer or "/", 302))
        LanguageSelector.set_language_cookie(locale, response)

        return response



import calendar
import re
from io import StringIO

from flask import current_app

from fsd_utils.mapping.application.languages import EN, NO, NOT_PROVIDED, SUPPORTED_LANGUAGES, YES


def convert_bool_value(data, language=EN):
    if not language and language not in SUPPORTED_LANGUAGES:
        language = EN

    try:

        def convert_values(value):
            if value is None or value == "None":
                return NOT_PROVIDED[language]
            if isinstance(value, bool):
                return YES[language] if value else NO[language]
            else:
                return value

        if isinstance(data, list):
            if all(isinstance(sublist, list) for sublist in data):
                converted_data = [[convert_values(value) for value in sublist] for sublist in data]
            else:
                converted_data = [convert_values(value) for value in data]
        else:
            converted_data = convert_values(data)

        return converted_data
    except Exception as e:
        current_app.logger.error("Could not convert boolean values, %s", str(e))


def format_answer(answer, language):
    try:
        if answer is None or answer == "None":
            return NOT_PROVIDED[language]

        if "null" in answer:
            return re.sub(r"\s*null\s*,?", "", answer)

        if isinstance(answer, list):
            return [a.replace("'", "") for a in answer if isinstance(a, str)]

        return answer
    except Exception:
        current_app.logger.info("No formatting required for an answer: %s", answer)


def simplify_title(section_name: str, remove_text: list) -> list:
    """
    Simplify a section title by removing specified text elements.

    This function takes a section title as input and removes specific text elements
    based on the provided 'remove_text' list. It splits the section title by hyphens
    and removes elements from the beginning until it encounters the first match in
    'remove_text'. If no match is found, the original section title is returned.

    Parameters:
    - section_name (str): The original section title to be simplified.
    - remove_text (list): A list of text elements to be removed from the title.

    Returns:
    - simplified_title (list): A list containing the simplified section title
      after removing the specified elements.
    """
    try:
        section = section_name.split("-")
        simplified_title = []

        for i, text in enumerate(section):
            if text in remove_text:
                simplified_title = section[:i]
                break

        if not simplified_title:
            simplified_title = section

        return simplified_title
    except Exception as e:
        current_app.logger.warning("Could not simplify the section title, %s", e)


def format_checkbox(answer: list) -> str:
    """
    Format a list of elements into a checkbox-style list.

    This function takes a list of elements as input and formats them into a
    checkbox-style list with proper indentation and capitalization. It processes
    each element, splitting hyphenated elements if present, and returns the formatted
    checkbox-style list as a string.

    Parameters:
    - answer (list): The list of elements to be formatted.

    Returns:
    - formatted_list (str): The formatted checkbox-style list as a string.
    """
    formatted_elements = []
    indent = " " * 5
    for index, element in enumerate(answer, start=1):
        separator = f"{indent}." if index > 1 else "."
        if "-" in element:
            sub_elements = element.split("-")
            formatted_sub_elements = " ".join(sub_elements).strip().capitalize()
            formatted_elements.append(f"{separator} {formatted_sub_elements}")
        else:
            formatted_elements.append(f"{separator} {element.capitalize()}")

    return "\n".join(formatted_elements)


def format_radio_field(answer: str) -> str:
    """
    Format a radio field answer for better readability.

    This function takes an answer from a radio field and applies formatting to
    enhance its readability. It handles cases where the answer is a URL, a boolean,
    a list, or a string with hyphens. It capitalizes the first letter of each word
    and joins them with spaces.

    Parameters:
    - answer: The radio field answer to be formatted.

    Returns:
    - formatted_answer: The formatted answer for improved readability.
    """
    try:
        if answer is None or isinstance(answer, (bool, list)):
            return answer

        # Check if answer looks like a URL
        if answer.startswith("http://") or answer.startswith("https://"):
            return answer

        if "-" in answer:
            answer = answer.split("-")
            formatted_answer = " ".join(answer).strip().capitalize()
            return formatted_answer
        else:
            return answer.capitalize()

    except Exception:  # noqa
        current_app.logger.info("continue: the answer doesn't seem to be a radio field.")
        return answer


def generate_text_of_application(q_and_a: dict, fund_name: str, language=EN) -> str:
    """
    Generate a formatted text document for an application with questions and answers.

    This function takes a dictionary of questions and answers ('q_and_a') and the name
    of a fund ('fund_name'). It creates a formatted text document containing the fund
    name, section titles derived from 'q_and_a' keys, and corresponding questions and
    answers. Text is formatted with section titles, question and answer numbering, and
    specific text elements are simplified as specified. The resulting output is
    returned as a string.

    Parameters:
    - q_and_a (dict): A dictionary of questions and answers.
    - fund_name (str): The name of the fund.
    - language (str): "en" or "cy", the language to use.

    Returns:
    - application_text (str): A formatted text output for the application.
    """
    if not language and language not in SUPPORTED_LANGUAGES:
        language = EN

    output = StringIO()

    output.write(f"********* {fund_name} **********\n")

    for section_name, values in q_and_a.items():
        title = simplify_title(section_name, remove_text=["cof", "ns", "cyp", "dpi", "hsra"])
        output.write(f"\n* {' '.join(title).capitalize()}\n\n")
        for questions, answers in values.items():
            output.write(f"  Q) {questions}\n")
            output.write(f"  A) {format_answer(answers, language)}\n\n")
    return output.getvalue()


def number_to_month(number, iso_key):
    """Converts a month number to its corresponding month name."""
    try:
        if iso_key == "month":
            month_name = calendar.month_name[number]
            return month_name
        else:
            return number
    except IndexError:
        current_app.logger.warning("Invalid month number")
        return number


def format_month_year(answer):
    try:
        answer_text = answer.split("-")
        month = answer_text[0] if len(answer_text[0]) <= 2 else answer_text[1]
        month_name = calendar.month_name[int(month)]
        if month_name:
            year = answer_text[1] if len(answer_text[1]) == 4 else answer_text[0]
            return f"{month_name} {year}"
    except Exception as e:
        current_app.logger.warning("Invalid month-year formatting for answer: %s. Error: %s", answer, str(e))

    return answer


def format_date_month_year(answer):
    try:
        answer_text = answer.split("-")
        month = answer_text[1]
        month_name = calendar.month_name[int(month)]
        if month_name:
            date = answer_text[2] if len(answer_text[2]) <= 2 else answer_text[0]
            date = f"{'0' + date if len(date) == 1 else date}"
            year = answer_text[0] if len(answer_text[0]) == 4 else answer_text[2]
            return f"{date} {month_name} {year}"
    except Exception as e:
        current_app.logger.warning("Invalid date-month-year formatting for answer: %s. Error: %s", answer, str(e))
    return answer



from unittest.mock import ANY, Mock

from fsd_utils.locale_selector.set_lang import LanguageSelector


def test_set_lang(flask_test_client):
    mock_app = Mock()
    set_lang = LanguageSelector(mock_app)
    mock_app.add_url_rule.assert_called_with("/language/<locale>", view_func=ANY)
    with flask_test_client.application.test_request_context():
        response = set_lang.select_language("cy")
        response_cookie = response.headers.get("Set-Cookie")
        assert response_cookie is not None, "No cookie set for language"
        assert response_cookie.split(";")[0] == ("language" + "=cy")



from botocore.exceptions import BotoCoreError


class SQSExtendedClientException(BotoCoreError):
    """
    Base Class for all SQS Extended Client exceptions which occur
    after interacting with the SQS Extended Client.
    """

    def __init__(self, error_message):
        self.fmt = error_message
        super().__init__()


class ExceptionMessages:
    INVALID_ARGUMENTS_FOR_DELETE_MESSAGE = "Invalid number of arguments while calling delete_message."
    INVALID_MESSAGE_ATTRIBUTE_SIZE = (
        "Total size of message attributes is {0} bytes which is larger than the "
        "threshold of {1} bytes. Consider including the payload in the message body "
        "instead of the message attributes."
    )
    INVALID_NUMBER_OF_MESSAGE_ATTRIBUTES = (
        "Number of message attributes {0} exceeds the maximum allowed for " "large-payload messages {1}"
    )
    INVALID_ATTRIBUTE_NAME_PRESENT = "Message attribute name {0} is reserved for use by the SQS extended client. "
    INVALID_MESSAGE_BODY = "messageBody cannot be null or empty."
    INVALID_FORMAT_WHEN_RETRIEVING_STORED_S3_MESSAGES = "Invalid payload format for retrieving stored messages in S3"

    FAILED_DELETE_MESSAGE = "delete_object failed with status code {0}"
    FAILED_SUBMIT_MESSAGE = "submit_single_message failed with status code {0}"
    FAILED_RECEIVE_MESSAGE = "receive_messages failed with status code {0}"



# -*- coding: utf-8 -*-
from setuptools import find_packages, setup

package_data = {"": ["*"]}

setup_kwargs = {
    "packages": find_packages(),
    "package_data": package_data,
}

setup(**setup_kwargs)



import json
import unittest
from datetime import datetime
from unittest.mock import MagicMock, patch
from uuid import uuid4

import pytest

from fsd_utils.services.aws_extended_client import SQSExtendedClient
from fsd_utils.services.aws_sqs_extended_client_exception import (
    SQSExtendedClientException,
)


class TestSQSExtendedClient(unittest.TestCase):
    def setUp(self):
        # Create a mock Boto3 SQS client for testing without extended client behaviour
        self.sqs_client = MagicMock()
        self.sqs_client.list_queues.return_value = {"QueueUrls": ["url1", "url2"]}
        self.sqs_client.delete_queue.return_value = {}
        # Create an instance of SQSClient with the mock client
        self.sqs = SQSExtendedClient(
            aws_access_key_id="your_access_key",  # pragma: allowlist secret
            aws_secret_access_key="your_secret_key",  # pragma: allowlist secret
            logger=MagicMock(),
        )
        self.sqs.sqs_client = self.sqs_client

        # Create a mock Boto3 SQS client for testing with extended client behaviour
        self.s3_client = MagicMock()
        self.sqs_client.list_queues.return_value = {"QueueUrls": ["url1", "url2"]}
        self.sqs_client.delete_queue.return_value = {}
        # Create an instance of SQSClient with the mock client
        self.sqs_extended = SQSExtendedClient(
            aws_access_key_id="your_access_key",  # pragma: allowlist secret
            aws_secret_access_key="your_secret_key",  # pragma: allowlist secret
            large_payload_support="fsd_sqs_extended_helper",
            always_through_s3=True,
            delete_payload_from_s3=True,
            logger=MagicMock(),
        )
        self.sqs_extended.sqs_client = self.sqs_client
        self.sqs_extended.s3_client = self.s3_client

    def test_get_queues_without_extended_client_behaviour(self):
        # Test the get_queues method without a prefix
        queue_names = self.sqs.get_queues()
        self.assertEqual(queue_names, ["url1", "url2"])

    def test_get_queues_with_extended_client_behaviour(self):
        # Test the get_queues method without a prefix
        queue_names = self.sqs_extended.get_queues()
        self.assertEqual(queue_names, ["url1", "url2"])

    def test_submit_single_message_without_extended_client_behaviour(self):
        # Mock data & responses
        queue_url = "https://sqs.us-west-1.amazonaws.com/123456789012/test_queue"
        message = "{'key': 'value'}"
        message_group_id = "group_id"
        message_deduplication_id = "deduplication_id"
        expected_message_id = "test_message_id"

        self.sqs_client.send_message.return_value = {
            "MessageId": expected_message_id,
            "ResponseMetadata": {"HTTPStatusCode": 200},
        }

        with patch("fsd_utils.services.aws_extended_client.datetime") as mock_datetime:
            datetime_now = datetime(2023, 1, 1, 12, 0, 0)
            mock_datetime.now.return_value = datetime_now

            # call to the function
            actual_message_id = self.sqs.submit_single_message(
                queue_url,
                message,
                message_group_id=message_group_id,
                message_deduplication_id=message_deduplication_id,
            )

            # Assert responses
            self.assertEqual(actual_message_id, expected_message_id)
            self.sqs_client.send_message.assert_called_with(
                QueueUrl=queue_url,
                MessageBody=message,
                MessageAttributes={
                    "message_created_at": {
                        "StringValue": str(datetime_now),
                        "DataType": "String",
                    }
                },
                MessageGroupId=message_group_id,
                MessageDeduplicationId=message_deduplication_id,
            )

    def test_submit_single_message_with_extended_client_behaviour(self):
        # Mock data & responses
        queue_url = "https://sqs.us-west-1.amazonaws.com/123456789012/test_queue"
        message = "{'key': 'value'}"
        message_group_id = "group_id"
        message_deduplication_id = "deduplication_id"
        expected_message_id = "test_message_id"

        self.sqs_client.send_message.return_value = {
            "MessageId": expected_message_id,
            "ResponseMetadata": {"HTTPStatusCode": 200},
        }

        self.s3_client.put_object.return_value = {"ResponseMetadata": {"HTTPStatusCode": 200}}

        with patch("fsd_utils.services.aws_extended_client.datetime") as mock_datetime:
            with patch("fsd_utils.services.aws_sqs_extended_client_util.uuid4") as mock_uuid:
                uuid_val = uuid4()
                datetime_now = datetime(2023, 1, 1, 12, 0, 0)
                mock_datetime.now.return_value = datetime_now
                mock_uuid.return_value = uuid_val

                message_body = json.dumps(
                    [
                        "software.amazon.payloadoffloading.PayloadS3Pointer",
                        {
                            "s3BucketName": "fsd_sqs_extended_helper",
                            "s3Key": str(uuid_val),
                        },
                    ]
                )

                # call to the function
                actual_message_id = self.sqs_extended.submit_single_message(
                    queue_url,
                    message,
                    message_group_id=message_group_id,
                    message_deduplication_id=message_deduplication_id,
                )

                # Assert responses
                self.assertEqual(actual_message_id, expected_message_id)
                self.sqs_client.send_message.assert_called_with(
                    QueueUrl=queue_url,
                    MessageBody=message_body,
                    MessageAttributes={
                        "ExtendedPayloadSize": {
                            "DataType": "Number",
                            "StringValue": "16",
                        },
                        "message_created_at": {
                            "StringValue": str(datetime_now),
                            "DataType": "String",
                        },
                    },
                    MessageGroupId=message_group_id,
                    MessageDeduplicationId=message_deduplication_id,
                )

    def test_submit_single_message_with_extended_client_behaviour_with_s3key(self):
        # Mock data & responses
        queue_url = "https://sqs.us-west-1.amazonaws.com/123456789012/test_queue"
        message = "{'key': 'value'}"
        message_group_id = "group_id"
        message_deduplication_id = "deduplication_id"
        expected_message_id = "test_message_id"

        self.sqs_client.send_message.return_value = {
            "MessageId": expected_message_id,
            "ResponseMetadata": {"HTTPStatusCode": 200},
        }

        self.s3_client.put_object.return_value = {"ResponseMetadata": {"HTTPStatusCode": 200}}

        with patch("fsd_utils.services.aws_extended_client.datetime") as mock_datetime:
            with patch("fsd_utils.services.aws_sqs_extended_client_util.uuid4") as mock_uuid:
                uuid_val = uuid4()
                datetime_now = datetime(2023, 1, 1, 12, 0, 0)
                mock_datetime.now.return_value = datetime_now
                mock_uuid.return_value = uuid_val

                message_body = json.dumps(
                    [
                        "software.amazon.payloadoffloading.PayloadS3Pointer",
                        {
                            "s3BucketName": "fsd_sqs_extended_helper",
                            "s3Key": "notification/" + str(uuid_val),
                        },
                    ]
                )

                # call to the function
                actual_message_id = self.sqs_extended.submit_single_message(
                    queue_url,
                    message,
                    message_group_id=message_group_id,
                    message_deduplication_id=message_deduplication_id,
                    extra_attributes={
                        "S3Key": {
                            "StringValue": "notification",
                            "DataType": "String",
                        },
                    },
                )

                # Assert responses
                self.assertEqual(actual_message_id, expected_message_id)
                self.sqs_client.send_message.assert_called_with(
                    QueueUrl=queue_url,
                    MessageBody=message_body,
                    MessageAttributes={
                        "ExtendedPayloadSize": {
                            "DataType": "Number",
                            "StringValue": "16",
                        },
                        "S3Key": {"DataType": "String", "StringValue": "notification"},
                        "message_created_at": {
                            "StringValue": str(datetime_now),
                            "DataType": "String",
                        },
                    },
                    MessageGroupId=message_group_id,
                    MessageDeduplicationId=message_deduplication_id,
                )

    def test_submit_single_message_with_extended_client_behaviour_error_sqs(self):
        # Mock data & responses
        queue_url = "https://sqs.us-west-1.amazonaws.com/123456789012/test_queue"
        message = "{'key': 'value'}"
        message_group_id = "group_id"
        message_deduplication_id = "deduplication_id"
        expected_message_id = "test_message_id"

        self.sqs_client.send_message.return_value = {
            "MessageId": expected_message_id,
            "ResponseMetadata": {"HTTPStatusCode": 500},
        }

        self.s3_client.put_object.return_value = {"ResponseMetadata": {"HTTPStatusCode": 200}}

        with patch("fsd_utils.services.aws_extended_client.datetime") as mock_datetime:
            with patch("fsd_utils.services.aws_sqs_extended_client_util.uuid4") as mock_uuid:
                uuid_val = uuid4()
                datetime_now = datetime(2023, 1, 1, 12, 0, 0)
                mock_datetime.now.return_value = datetime_now
                mock_uuid.return_value = uuid_val

                with pytest.raises(
                    SQSExtendedClientException,
                    match="submit_single_message failed with status code 500",
                ):
                    # call to the function
                    self.sqs_extended.submit_single_message(
                        queue_url,
                        message,
                        message_group_id=message_group_id,
                        message_deduplication_id=message_deduplication_id,
                    )

    def test_submit_single_message_with_extended_client_behaviour_error_s3(self):
        # Mock data & responses
        queue_url = "https://sqs.us-west-1.amazonaws.com/123456789012/test_queue"
        message = "{'key': 'value'}"
        message_group_id = "group_id"
        message_deduplication_id = "deduplication_id"
        expected_message_id = "test_message_id"

        self.sqs_client.send_message.return_value = {
            "MessageId": expected_message_id,
            "ResponseMetadata": {"HTTPStatusCode": 200},
        }

        self.s3_client.put_object.return_value = {"ResponseMetadata": {"HTTPStatusCode": 500}}

        with pytest.raises(
            SQSExtendedClientException,
            match="submit_single_message failed with status code 500",
        ):
            # call to the function
            self.sqs_extended.submit_single_message(
                queue_url,
                message,
                message_group_id=message_group_id,
                message_deduplication_id=message_deduplication_id,
            )

    def test_receive_messages_without_extended_client_behaviour(self):
        # Mock data & responses
        queue_url = "http://localhost:4576/queue/test_queue"
        max_number = 5
        visibility_time = 10
        wait_time = 2
        messages = [
            {
                "MessageId": "msg1",
                "MessageAttributes": {"attr1": "value1"},
                "MessageBody": "Message 1",
            },
            {
                "MessageId": "msg2",
                "MessageAttributes": {"attr2": "value2"},
                "MessageBody": "Message 2",
            },
        ]
        response = {"Messages": messages, "ResponseMetadata": {"HTTPStatusCode": 200}}
        self.sqs_client.receive_message.return_value = response

        # call to the function
        received_messages = self.sqs.receive_messages(queue_url, max_number, visibility_time, wait_time)

        # Assert responses
        self.assertEqual(received_messages, messages)
        self.sqs_client.receive_message.assert_called_with(
            QueueUrl=queue_url,
            AttributeNames=["SentTimestamp", "ApproximateReceiveCount"],
            MessageAttributeNames=["All"],
            MaxNumberOfMessages=max_number,
            VisibilityTimeout=visibility_time,
            WaitTimeSeconds=wait_time,
        )

    def test_receive_messages_with_extended_client_behaviour(self):
        # Mock data & responses
        uuid_val = uuid4()
        queue_url = "http://localhost:4576/queue/test_queue"
        max_number = 5
        visibility_time = 10
        wait_time = 2
        message_body = json.dumps(
            [
                "software.amazon.payloadoffloading.PayloadS3Pointer",
                {"s3BucketName": "fsd_sqs_extended_helper", "s3Key": str(uuid_val)},
            ]
        )
        messages = [
            {
                "MessageId": "msg1",
                "MessageAttributes": {"attr1": "value1"},
                "Body": message_body,
            }
        ]
        response = {"Messages": messages, "ResponseMetadata": {"HTTPStatusCode": 200}}
        body_data = MagicMock()
        body_data.read.decode = "Testing"
        s3_response = {"Body": body_data, "ResponseMetadata": {"HTTPStatusCode": 200}}
        s3_response["Body"].read.decode = "Testing"
        self.sqs_client.receive_message.return_value = response
        self.s3_client.get_object.return_value = s3_response

        # call to the function
        received_messages = self.sqs_extended.receive_messages(queue_url, max_number, visibility_time, wait_time)

        # Assert responses
        self.assertEqual(received_messages[0]["sqs"], messages[0])
        self.sqs_client.receive_message.assert_called_with(
            QueueUrl=queue_url,
            AttributeNames=["SentTimestamp", "ApproximateReceiveCount"],
            MessageAttributeNames=["All"],
            MaxNumberOfMessages=max_number,
            VisibilityTimeout=visibility_time,
            WaitTimeSeconds=wait_time,
        )

    def test_receive_messages_with_extended_client_behaviour_error_sqs(self):
        # Mock data & responses
        uuid_val = uuid4()
        queue_url = "http://localhost:4576/queue/test_queue"
        max_number = 5
        visibility_time = 10
        wait_time = 2
        message_body = json.dumps(
            [
                "software.amazon.payloadoffloading.PayloadS3Pointer",
                {"s3BucketName": "fsd_sqs_extended_helper", "s3Key": str(uuid_val)},
            ]
        )
        messages = [
            {
                "MessageId": "msg1",
                "MessageAttributes": {"attr1": "value1"},
                "Body": message_body,
            }
        ]
        response = {"Messages": messages, "ResponseMetadata": {"HTTPStatusCode": 500}}
        body_data = MagicMock()
        body_data.read.decode = "Testing"
        s3_response = {"Body": body_data, "ResponseMetadata": {"HTTPStatusCode": 200}}
        s3_response["Body"].read.decode = "Testing"
        self.sqs_client.receive_message.return_value = response
        self.s3_client.get_object.return_value = s3_response

        # call to the function
        with pytest.raises(
            SQSExtendedClientException,
            match="receive_messages failed with status code 500",
        ):
            self.sqs_extended.receive_messages(queue_url, max_number, visibility_time, wait_time)

    def test_receive_messages_with_extended_client_behaviour_error_s3(self):
        # Mock data & responses
        uuid_val = uuid4()
        queue_url = "http://localhost:4576/queue/test_queue"
        max_number = 5
        visibility_time = 10
        wait_time = 2
        message_body = json.dumps(
            [
                "software.amazon.payloadoffloading.PayloadS3Pointer",
                {"s3BucketName": "fsd_sqs_extended_helper", "s3Key": str(uuid_val)},
            ]
        )
        messages = [
            {
                "MessageId": "msg1",
                "MessageAttributes": {"attr1": "value1"},
                "Body": message_body,
            }
        ]
        response = {"Messages": messages, "ResponseMetadata": {"HTTPStatusCode": 200}}
        s3_response = {"Body": "eee", "ResponseMetadata": {"HTTPStatusCode": 500}}
        self.sqs_client.receive_message.return_value = response
        self.s3_client.get_object.return_value = s3_response

        # call to the function
        with pytest.raises(
            SQSExtendedClientException,
            match="receive_messages failed with status code 500",
        ):
            self.sqs_extended.receive_messages(queue_url, max_number, visibility_time, wait_time)

    def test_delete_messages_without_extended_client_behaviour(self):
        # Mock data & responses
        queue_url = "http://localhost:4576/queue/test_queue"
        message_receipt_handles = [
            {
                "MessageId": "msg_id_1",
                "ReceiptHandle": "receipt_handle1",
                "Body": "body 1",
            },
            {
                "MessageId": "msg_id_2",
                "ReceiptHandle": "receipt_handle2",
                "Body": "body 2",
            },
            {
                "MessageId": "msg_id_3",
                "ReceiptHandle": "receipt_handle3",
                "Body": "body 3",
            },
        ]
        response = {
            "Successful": [{"Id": "0"}, {"Id": "1"}],
            "Failed": [{"Id": "2"}],
            "ResponseMetadata": {"HTTPStatusCode": 200},
        }
        self.sqs_client.delete_message_batch.return_value = response

        # call to the function
        deleted_response = self.sqs.delete_messages(queue_url, message_receipt_handles)

        # Assert responses
        self.assertEqual(deleted_response, response)
        expected_entries = [
            {"Id": "0", "ReceiptHandle": "receipt_handle1"},
            {"Id": "1", "ReceiptHandle": "receipt_handle2"},
            {"Id": "2", "ReceiptHandle": "receipt_handle3"},
        ]
        self.sqs_client.delete_message_batch.assert_called_with(QueueUrl=queue_url, Entries=expected_entries)

    def test_delete_messages_with_extended_client_behaviour(self):
        uuid_val = uuid4()
        # Mock data & responses
        queue_url = "http://localhost:4576/queue/test_queue"
        message_body = json.dumps(
            [
                "software.amazon.payloadoffloading.PayloadS3Pointer",
                {"s3BucketName": "fsd_sqs_extended_helper", "s3Key": str(uuid_val)},
            ]
        )
        message_receipt_handles = [
            {
                "MessageId": "msg_id_1",
                "ReceiptHandle": "receipt_handle1",
                "Body": message_body,
            }
        ]
        response = {
            "Successful": [{"Id": "0"}],
            "ResponseMetadata": {"HTTPStatusCode": 200},
        }
        s3_response = {"ResponseMetadata": {"HTTPStatusCode": 204}}
        self.s3_client.delete_object.return_value = s3_response
        self.sqs_client.delete_message_batch.return_value = response

        # call to the function
        deleted_response = self.sqs_extended.delete_messages(queue_url, message_receipt_handles)

        # Assert responses
        self.assertEqual(deleted_response, response)
        expected_entries = [{"Id": "0", "ReceiptHandle": "receipt_handle1"}]
        self.sqs_client.delete_message_batch.assert_called_with(QueueUrl=queue_url, Entries=expected_entries)

    def test_delete_messages_with_extended_client_behaviour_error_sqs(self):
        uuid_val = uuid4()
        # Mock data & responses
        queue_url = "http://localhost:4576/queue/test_queue"
        message_body = json.dumps(
            [
                "software.amazon.payloadoffloading.PayloadS3Pointer",
                {"s3BucketName": "fsd_sqs_extended_helper", "s3Key": str(uuid_val)},
            ]
        )
        message_receipt_handles = [
            {
                "MessageId": "msg_id_1",
                "ReceiptHandle": "receipt_handle1",
                "Body": message_body,
            }
        ]
        response = {
            "Successful": [{"Id": "0"}],
            "ResponseMetadata": {"HTTPStatusCode": 500},
        }
        s3_response = {"ResponseMetadata": {"HTTPStatusCode": 204}}
        self.s3_client.delete_object.return_value = s3_response
        self.sqs_client.delete_message_batch.return_value = response

        # call to the function
        with pytest.raises(
            SQSExtendedClientException,
            match="delete_object failed with status code 500",
        ):
            self.sqs_extended.delete_messages(queue_url, message_receipt_handles)

    def test_delete_messages_with_extended_client_behaviour_error_s3(self):
        uuid_val = uuid4()
        # Mock data & responses
        queue_url = "http://localhost:4576/queue/test_queue"
        message_body = json.dumps(
            [
                "software.amazon.payloadoffloading.PayloadS3Pointer",
                {"s3BucketName": "fsd_sqs_extended_helper", "s3Key": str(uuid_val)},
            ]
        )
        message_receipt_handles = [
            {
                "MessageId": "msg_id_1",
                "ReceiptHandle": "receipt_handle1",
                "Body": message_body,
            }
        ]
        response = {
            "Successful": [{"Id": "0"}],
            "ResponseMetadata": {"HTTPStatusCode": 200},
        }
        s3_response = {"ResponseMetadata": {"HTTPStatusCode": 500}}
        self.s3_client.delete_object.return_value = s3_response
        self.sqs_client.delete_message_batch.return_value = response

        # call to the function
        with pytest.raises(
            SQSExtendedClientException,
            match="delete_object failed with status code 500",
        ):
            self.sqs_extended.delete_messages(queue_url, message_receipt_handles)
